# BEGIN COPYRIGHT
# *************************************************************************
#
# IBM Confidential
# OCO Source Materials
# These are as-is scripts given only as a guide.
# 5724-M24
# (C) Copyright IBM Corporation 2008
# The source code for this program is not published or otherwise
# divested of its trade secrets, irrespective of what has been
# deposited with the U.S. Copyright Office.
#
# *************************************************************************
# END COPYRIGHT

#This script can be called from a command-line wrapper script or from the profile upgrade framework.
#If called from the framework, then the System properties will hold the relevant parameters.
#If called from the command-line then the expected parameters are either: 
#[wasHome, clusterName] or [wasHome, nodeName, serverName]


from java.lang import System
import java.io.File as File;
import zipfile
import shutil

#*******************************************************************************************************
#This function implements the profile upgrade framework definition that accepts
#an applicationName, nodeName, serverName, and clusterName).  It will be called by the
#framework function updateDeploymentTargetsOf.  Either the nodeName/serverName pair will be specified
#or the clusterName parameter will be specified.  The others will be None
#wasHome is expected to be previously defined
#*******************************************************************************************************
def upgradeBSpaceProfile(appName, nodeName, serverName, clusterName):

  if ( clusterName is not None ):
    appSubfixName = '_' + clusterName
    updateScopeArgs = '-clusterName "' + clusterName + '"'
    mmRuntimeProfPath = profileRoot + '/BusinessSpace/' + clusterName + '/mm.runtime.prof'
  elif ( (nodeName is not None) and (serverName is not None) ):
    appSubfixName = '_' + nodeName + '_' + serverName
    updateScopeArgs = '-serverName "' + serverName + '" -nodeName "' + nodeName + '"'
    mmRuntimeProfPath = profileRoot + '/BusinessSpace/' + nodeName + '/' + serverName+ '/mm.runtime.prof'
  #endif

  #get the version numbers of the deployed and installed ears for comparison
  deployedVersion = getDeployedVersionId(appSubfixName)
  upgradeVersion = getIntalledVersionId()

  #check to see if the profile should be upgraded
  upgradeProfileFlag = 1;
  if ( (deployedVersion != "") and (upgradeVersion != "") ):
    upgradeProfileFlag = int(deployedVersion) < int(upgradeVersion)

  print "Upgrade the profile?: " + repr(upgradeProfileFlag)
  
  if (upgradeProfileFlag):
    #Update the EARs
    AdminApp.update('mm.was' + appSubfixName,'app','[-operation update -contents "' + wasHome + '/installableApps/BusinessSpace/mm.was.ear"]')
    AdminApp.update('BSpaceEAR' + appSubfixName,'app','[-operation update -contents "' + wasHome + '/installableApps/BusinessSpace/BSpaceEAR.ear"]')
    AdminApp.update('BSpaceWebformsEnabler' + appSubfixName,'app','[-operation update -contents "' + wasHome + '/installableApps/BusinessSpace/BSpaceWebformsEnabler.ear"]')
    AdminApp.update('BusinessSpaceHelpEAR' + appSubfixName,'app','[-operation update -contents "' + wasHome + '/installableApps/BusinessSpace/BusinessSpaceHelpEAR.ear"]')
    
    #d640537: set authenticated
    AdminApp.edit('mm.was' + appSubfixName, '[-MapRolesToUsers [["Allauthenticated" AppDeploymentOption.No AppDeploymentOption.Yes "" ""]]]')

    #Update widgets
    updateWidgetsArgs = '[' + updateScopeArgs  
    updateWidgetsArgs = updateWidgetsArgs + ' -widgets "' +wasHome + '/BusinessSpace/widgets/CORE/widget_bspace_core.zip"' 
    updateWidgetsArgs = updateWidgetsArgs + ' -save "false"'  + ' -noEndpoints "true"' + ']'
    AdminTask.updateBusinessSpaceWidgets(updateWidgetsArgs)
    
    #Update ContextMenu
    srcContextMenuFile = wasHome + '/BusinessSpace/mm.runtime/ContextMenu_Provider/contextmenu.xml'
    destContextMenuPath = mmRuntimeProfPath + '/ContextMenu_Provider'
    shutil.copy(srcContextMenuFile,destContextMenuPath )

    updateBlobConfigArgs = '[' + updateScopeArgs
    updateBlobConfigArgs = updateBlobConfigArgs + ' -propertyDirectoryName "' + destContextMenuPath + '"' 
    updateBlobConfigArgs = updateBlobConfigArgs + ' -suffix "ContextMenu_Provider"' + ']'
    AdminTask.updateBlobConfig(updateBlobConfigArgs)
    
    #Update ConfigService
    configServiceFileName = mmRuntimeProfPath + '/config/ConfigService.properties'
    newProperties = '\ncom.ibm.mashups.url.webdav = /mm/dav/filestore/'
    newProperties = newProperties + '\ncom.ibm.mashups.autoWildcardSearchDisabled=false'
    newProperties = newProperties + '\ncom.ibm.mashups.user.userProfile=ALL'
    newProperties = newProperties + '\ncom.ibm.mashups.user.groupProfile=ALL'
    newProperties = newProperties + '\ncom.ibm.mashups.lockeddown = false'
    f = open(configServiceFileName, "a")
    f.write(newProperties)
    f.close()

    updatePropertyConfigArgs = '[' + updateScopeArgs
    updatePropertyConfigArgs = updatePropertyConfigArgs + ' -propertyDirectoryName "' + mmRuntimeProfPath + '/config"' + ']'
    AdminTask.updatePropertyConfig(updatePropertyConfigArgs)

    #Update default template
    srcTemplateFile = wasHome + '/BusinessSpace/mm.runtime/templates/defaultTemplate.zip'
    destTemplatePath = mmRuntimeProfPath + '/templates'
    shutil.copy(srcTemplateFile,destTemplatePath)

    updateTemplateArgs = '[' + updateScopeArgs
    updateTemplateArgs = updateTemplateArgs + ' -templates "' + destTemplatePath + '/defaultTemplate.zip"' 
    updateTemplateArgs = updateTemplateArgs + ' -save "false"'  + ']'
    AdminTask.updateBusinessSpaceWidgets(updateTemplateArgs)
    
    #save
    AdminConfig.save()
  else:
    #The profile upgrade has already completed for this target, no need to rerun.
    print "The profile upgrade is not necessary"
  #endif
#enddef

#*******************************************************************************************************
#This function returns the version string of the deployed BSpace ear
#*******************************************************************************************************
def getDeployedVersionId(appSubfixName):
  #Find and return the BSpace version number of the deployed BSpace app 
  deployedVersion = "";
  versionArray = AdminApp.view('BSpaceEAR' + appSubfixName,'[-buildVersion]').split()
  if ((versionArray is not None) and (len(versionArray) >= 4) and (versionArray[3] is not None)):
    deployedVersion = stripVersion(versionArray[3])
    print "Deployed version of the Business Space EAR:  " + deployedVersion
  #endif
  return deployedVersion
#enddef

#*******************************************************************************************************
#This function returns the version string of the newly installed BSpace ear
#*******************************************************************************************************
def getIntalledVersionId():
  #Find the version of the BSpace EAR in installableApps for comparison
  #This is stored in the MANIFEST.MF file
  upgradeVersion = ""
  bSpaceEARfile = zipfile.ZipFile(wasHome + '//installableApps/BusinessSpace/BSpaceEAR.ear')
  manifestFileData = bSpaceEARfile.read('META-INF/MANIFEST.MF')
  lines = manifestFileData.splitlines()
  for line in lines:
    if (line.startswith('Implementation-Version')):
      versionLine = line.split(':')
      if ((versionLine is not None) and (len(versionLine) == 2)):
        upgradeVersion = stripVersion(versionLine[1].lstrip())
        print "Installed version of the Business Space EAR: " + upgradeVersion
      #endif
    #endif
  #endfor
  return upgradeVersion
#enddef

#*******************************************************************************************************
#This function takes a full BSpace version string [product number]-[YYYYMMDD]_[HHMM] and returns, just the
#date part of the version id
#*******************************************************************************************************
def stripVersion(fullVersionId):
  version = ""
  lIndex = fullVersionId.rfind("-")
  rIndex = fullVersionId.find("_")
  if (lIndex >=0 and rIndex >= 0):
    version = fullVersionId[lIndex+1:rIndex]
  #endif
  if (version == ""):
    print "Warning: Can not determine version"
  #endif
  return version
#enddef

#*******************************************************************************************************
#This function converts path to unix style
#*******************************************************************************************************
def convertPathToUnixStyle(pathName):
  newPath = File(pathName);
  newPath = newPath.getAbsolutePath()
  return newPath.replace('\\',"/");
#endDef

#Main

#initialize the variables
wasHome = None
clusterName = None
appSubfixName = None
nodeName = None
serverName = None
profileRoot = System.getProperty("user.install.root")
profileRoot = convertPathToUnixStyle(profileRoot);

#nodeType will be in the System properties if this script is called via the upgrade framework
nodeType = System.getProperty("bpm.upgrade.node.type")

print

if (nodeType is None):  #script called from command line
  if ((len(sys.argv) == 2)):
    wasHome = sys.argv[0]
    clusterName = sys.argv[1]
  elif ((len(sys.argv) == 3)):
    wasHome = sys.argv[0]
    nodeName = sys.argv[1]
    serverName= sys.argv[2]
  else:
    print "Incorrect Parameters"
    sys.exit(-1)
  #endif
  upgradeBSpaceProfile('BSpaceEAR', nodeName, serverName, clusterName)

else:	#script called from upgrade framework
  wasHome = System.getProperty("was.install.root")

  #Use the framework's "updateDeploymentTargetsOf" function to invoke
  #the above callable (function) for each deployment target where an
  #application whose name starts with "BSpaceEAR" is deployed.
  appNames = [appName for appName in wsadminToList(AdminApp.list()) if appName.startswith("BSpaceEAR") ]
  updateDeploymentTargetsOf(appNames, upgradeBSpaceProfile)  
#endif

print

#end Main